@@ -26,6 +26,7 @@ from osc_lib.cli import parseractions
 from osc_lib.command import command
 from osc_lib import exceptions
 from osc_lib import utils
+from oslo_utils import timeutils
 import six
 
 try:
@@ -113,7 +114,7 @@ def _get_ip_address(addresses, address_type, ip_address_family):
         )
 
 
-def _prep_server_detail(compute_client, server):
+def _prep_server_detail(compute_client, image_client, server):
     """Prepare the detailed server dict for printing
 
     :param compute_client: a compute client instance
@@ -130,7 +131,7 @@ def _prep_server_detail(compute_client, server):
     if image_info:
         image_id = image_info.get('id', '')
         try:
-            image = utils.find_resource(compute_client.images, image_id)
+            image = utils.find_resource(image_client.images, image_id)
             info['image'] = "%s (%s)" % (image.name, image_id)
         except Exception:
             info['image'] = image_id
@@ -144,6 +145,20 @@ def _prep_server_detail(compute_client, server):
     except Exception:
         info['flavor'] = flavor_id
 
+    if 'os-extended-volumes:volumes_attached' in info:
+        info.update(
+            {
+                'volumes_attached': utils.format_list_of_dicts(
+                    info.pop('os-extended-volumes:volumes_attached'))
+            }
+        )
+    if 'security_groups' in info:
+        info.update(
+            {
+                'security_groups': utils.format_list_of_dicts(
+                    info.pop('security_groups'))
+            }
+        )
     # NOTE(dtroyer): novaclient splits these into separate entries...
     # Format addresses in a useful way
     info['addresses'] = _format_servers_list_networks(server.networks)
@@ -321,12 +336,15 @@ class CreateServer(command.ShowOne):
         disk_group.add_argument(
             '--image',
             metavar='<image>',
-            help=_('Create server from this image (name or ID)'),
+            help=_('Create server boot disk from this image (name or ID)'),
         )
         disk_group.add_argument(
             '--volume',
             metavar='<volume>',
-            help=_('Create server from this volume (name or ID)'),
+            help=_(
+                'Create server using this volume as the boot disk '
+                '(name or ID)'
+            ),
         )
         parser.add_argument(
             '--flavor',
@@ -436,12 +454,13 @@ class CreateServer(command.ShowOne):
     def take_action(self, parsed_args):
         compute_client = self.app.client_manager.compute
         volume_client = self.app.client_manager.volume
+        image_client = self.app.client_manager.image
 
         # Lookup parsed_args.image
         image = None
         if parsed_args.image:
             image = utils.find_resource(
-                compute_client.images,
+                image_client.images,
                 parsed_args.image,
             )
 
@@ -492,28 +511,40 @@ class CreateServer(command.ShowOne):
                            "exception": e}
                 )
 
-        block_device_mapping = {}
+        block_device_mapping_v2 = []
         if volume:
-            # When booting from volume, for now assume no other mappings
-            # This device value is likely KVM-specific
-            block_device_mapping = {'vda': volume}
-        else:
-            for dev_map in parsed_args.block_device_mapping:
-                dev_key, dev_vol = dev_map.split('=', 1)
-                block_volume = None
-                if dev_vol:
-                    vol = dev_vol.split(':', 1)[0]
-                    if vol:
-                        vol_id = utils.find_resource(
+            block_device_mapping_v2 = [{'uuid': volume,
+                                        'boot_index': '0',
+                                        'source_type': 'volume',
+                                        'destination_type': 'volume'
+                                        }]
+        for dev_map in parsed_args.block_device_mapping:
+            dev_name, dev_map = dev_map.split('=', 1)
+            if dev_map:
+                dev_map = dev_map.split(':')
+                if len(dev_map) > 0:
+                    mapping = {
+                        'device_name': dev_name,
+                        'uuid': utils.find_resource(
                             volume_client.volumes,
-                            vol,
-                        ).id
-                        block_volume = dev_vol.replace(vol, vol_id)
+                            dev_map[0],
+                        ).id}
+                    # Block device mapping v1 compatibility
+                    if len(dev_map) > 1 and \
+                            dev_map[1] in ('volume', 'snapshot'):
+                        mapping['source_type'] = dev_map[1]
                     else:
-                        msg = _("Volume name or ID must be specified if "
-                                "--block-device-mapping is specified")
-                        raise exceptions.CommandError(msg)
-                block_device_mapping.update({dev_key: block_volume})
+                        mapping['source_type'] = 'volume'
+                    mapping['destination_type'] = 'volume'
+                    if len(dev_map) > 2:
+                        mapping['volume_size'] = dev_map[2]
+                    if len(dev_map) > 3:
+                        mapping['delete_on_termination'] = dev_map[3]
+                else:
+                    msg = _("Volume name or ID must be specified if "
+                            "--block-device-mapping is specified")
+                    raise exceptions.CommandError(msg)
+                block_device_mapping_v2.append(mapping)
 
         nics = []
         if parsed_args.nic in ('auto', 'none'):
@@ -583,7 +614,7 @@ class CreateServer(command.ShowOne):
             userdata=userdata,
             key_name=parsed_args.key_name,
             availability_zone=parsed_args.availability_zone,
-            block_device_mapping=block_device_mapping,
+            block_device_mapping_v2=block_device_mapping_v2,
             nics=nics,
             scheduler_hints=hints,
             config_drive=config_drive)
@@ -615,7 +646,7 @@ class CreateServer(command.ShowOne):
                 sys.stdout.write(_('Error creating server\n'))
                 raise SystemExit
 
-        details = _prep_server_detail(compute_client, server)
+        details = _prep_server_detail(compute_client, image_client, server)
         return zip(*sorted(six.iteritems(details)))
 
 
@@ -778,11 +809,26 @@ class ListServer(command.Lister):
                    " 'osapi_max_limit' option of Nova API,"
                    " 'osapi_max_limit' will be used instead."),
         )
+        parser.add_argument(
+            '--deleted',
+            action="store_true",
+            default=False,
+            help=_('Only display deleted servers (Admin only).')
+        )
+        parser.add_argument(
+            '--changes-since',
+            metavar='<changes-since>',
+            default=None,
+            help=_("List only servers changed after a certain point of time."
+                   " The provided time should be an ISO 8061 formatted time."
+                   " ex 2016-03-04T06:27:59Z .")
+        )
         return parser
 
     def take_action(self, parsed_args):
         compute_client = self.app.client_manager.compute
         identity_client = self.app.client_manager.identity
+        image_client = self.app.client_manager.image
 
         project_id = None
         if parsed_args.project:
@@ -812,7 +858,7 @@ class ListServer(command.Lister):
         # image name is given, map it to ID.
         image_id = None
         if parsed_args.image:
-            image_id = utils.find_resource(compute_client.images,
+            image_id = utils.find_resource(image_client.images,
                                            parsed_args.image).id
 
         search_opts = {
@@ -828,9 +874,19 @@ class ListServer(command.Lister):
             'tenant_id': project_id,
             'all_tenants': parsed_args.all_projects,
             'user_id': user_id,
+            'deleted': parsed_args.deleted,
+            'changes_since': parsed_args.changes_since,
         }
         LOG.debug('search options: %s', search_opts)
 
+        if search_opts['changes_since']:
+            try:
+                timeutils.parse_isotime(search_opts['changes_since'])
+            except ValueError:
+                raise exceptions.CommandError(_('Invalid changes-since value:'
+                                              ' %s') % search_opts['changes'
+                                                                   '_since'])
+
         if parsed_args.long:
             columns = (
                 'ID',
@@ -1150,13 +1206,14 @@ class RebuildServer(command.ShowOne):
 
     def take_action(self, parsed_args):
         compute_client = self.app.client_manager.compute
+        image_client = self.app.client_manager.image
 
         server = utils.find_resource(
             compute_client.servers, parsed_args.server)
 
         # If parsed_args.image is not set, default to the currently used one.
         image_id = parsed_args.image or server._info.get('image', {}).get('id')
-        image = utils.find_resource(compute_client.images, image_id)
+        image = utils.find_resource(image_client.images, image_id)
 
         server = server.rebuild(image, parsed_args.password)
         if parsed_args.wait:
@@ -1172,7 +1229,7 @@ class RebuildServer(command.ShowOne):
                 sys.stdout.write(_('Error rebuilding server\n'))
                 raise SystemExit
 
-        details = _prep_server_detail(compute_client, server)
+        details = _prep_server_detail(compute_client, image_client, server)
         return zip(*sorted(six.iteritems(details)))
 
 
@@ -1547,7 +1604,8 @@ class ShowServer(command.ShowOne):
                 sys.stderr.write(_("Error retrieving diagnostics data\n"))
                 return ({}, {})
         else:
-            data = _prep_server_detail(compute_client, server)
+            data = _prep_server_detail(compute_client,
+                                       self.app.client_manager.image, server)
 
         return zip(*sorted(six.iteritems(data)))
 
