@@ -20,6 +20,7 @@ import logging
 from keystoneauth1 import exceptions as ks_exc
 from osc_lib.cli import parseractions
 from osc_lib.command import command
+from osc_lib import exceptions
 from osc_lib import utils
 import six
 
@@ -30,7 +31,7 @@ LOG = logging.getLogger(__name__)
 
 
 class CreateProject(command.ShowOne):
-    """Create new project"""
+    _description = _("Create new project")
 
     def get_parser(self, prog_name):
         parser = super(CreateProject, self).get_parser(prog_name)
@@ -102,7 +103,7 @@ class CreateProject(command.ShowOne):
 
 
 class DeleteProject(command.Command):
-    """Delete project(s)"""
+    _description = _("Delete project(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteProject, self).get_parser(prog_name)
@@ -117,16 +118,29 @@ class DeleteProject(command.Command):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
+        errors = 0
         for project in parsed_args.projects:
-            project_obj = utils.find_resource(
-                identity_client.tenants,
-                project,
-            )
-            identity_client.tenants.delete(project_obj.id)
+            try:
+                project_obj = utils.find_resource(
+                    identity_client.tenants,
+                    project,
+                )
+                identity_client.tenants.delete(project_obj.id)
+            except Exception as e:
+                errors += 1
+                LOG.error(_("Failed to delete project with "
+                          "name or ID '%(project)s': %(e)s"),
+                          {'project': project, 'e': e})
+
+        if errors > 0:
+            total = len(parsed_args.projects)
+            msg = (_("%(errors)s of %(total)s projects failed "
+                   "to delete.") % {'errors': errors, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListProject(command.Lister):
-    """List projects"""
+    _description = _("List projects")
 
     def get_parser(self, prog_name):
         parser = super(ListProject, self).get_parser(prog_name)
@@ -152,7 +166,7 @@ class ListProject(command.Lister):
 
 
 class SetProject(command.Command):
-    """Set project properties"""
+    _description = _("Set project properties")
 
     def get_parser(self, prog_name):
         parser = super(SetProject, self).get_parser(prog_name)
@@ -221,7 +235,7 @@ class SetProject(command.Command):
 
 
 class ShowProject(command.ShowOne):
-    """Display project details"""
+    _description = _("Display project details")
 
     def get_parser(self, prog_name):
         parser = super(ShowProject, self).get_parser(prog_name)
@@ -279,7 +293,7 @@ class ShowProject(command.ShowOne):
 
 
 class UnsetProject(command.Command):
-    """Unset project properties"""
+    _description = _("Unset project properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetProject, self).get_parser(prog_name)
