@@ -29,6 +29,7 @@ import six
 from openstackclient.i18n import _
 from openstackclient.identity import common as identity_common
 from openstackclient.network import common
+from openstackclient.network import sdk_utils
 from openstackclient.network import utils as network_utils
 
 
@@ -50,28 +51,27 @@ def _format_network_port_range(rule):
     # - Single port: '80:80'
     # - No port range: ''
     port_range = ''
-    if _is_icmp_protocol(rule.protocol):
-        if rule.port_range_min:
-            port_range += 'type=' + str(rule.port_range_min)
-        if rule.port_range_max:
-            port_range += ':code=' + str(rule.port_range_max)
-    elif rule.port_range_min or rule.port_range_max:
-        port_range_min = str(rule.port_range_min)
-        port_range_max = str(rule.port_range_max)
-        if rule.port_range_min is None:
+    if _is_icmp_protocol(rule['protocol']):
+        if rule['port_range_min']:
+            port_range += 'type=' + str(rule['port_range_min'])
+        if rule['port_range_max']:
+            port_range += ':code=' + str(rule['port_range_max'])
+    elif rule['port_range_min'] or rule['port_range_max']:
+        port_range_min = str(rule['port_range_min'])
+        port_range_max = str(rule['port_range_max'])
+        if rule['port_range_min'] is None:
             port_range_min = port_range_max
-        if rule.port_range_max is None:
+        if rule['port_range_max'] is None:
             port_range_max = port_range_min
         port_range = port_range_min + ':' + port_range_max
     return port_range
 
 
 def _get_columns(item):
-    columns = list(item.keys())
-    if 'tenant_id' in columns:
-        columns.remove('tenant_id')
-        columns.append('project_id')
-    return tuple(sorted(columns))
+    column_map = {
+        'tenant_id': 'project_id',
+    }
+    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
 
 
 def _convert_to_lowercase(string):
@@ -89,6 +89,8 @@ def _is_icmp_protocol(protocol):
         return False
 
 
+# TODO(abhiraut): Use the SDK resource mapped attribute names once the
+# OSC minimum requirements include SDK 1.0.
 class CreateSecurityGroupRule(common.NetworkAndComputeShowOne):
     _description = _("Create a new security group rule")
 
@@ -341,9 +343,9 @@ class CreateSecurityGroupRule(common.NetworkAndComputeShowOne):
 
         # Create and show the security group rule.
         obj = client.create_security_group_rule(**attrs)
-        columns = _get_columns(obj)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns)
-        return (columns, data)
+        return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
         group = utils.find_resource(
@@ -421,6 +423,16 @@ class DeleteSecurityGroupRule(common.NetworkAndComputeDelete):
 class ListSecurityGroupRule(common.NetworkAndComputeLister):
     _description = _("List security group rules")
 
+    def _format_network_security_group_rule(self, rule):
+        """Transform the SDK SecurityGroupRule object to a dict
+
+        The SDK object gets in the way of reformatting columns...
+        Create port_range column from port_range_min and port_range_max
+        """
+        rule = rule.to_dict()
+        rule['port_range'] = _format_network_port_range(rule)
+        return rule
+
     def update_parser_common(self, parser):
         parser.add_argument(
             'group',
@@ -506,7 +518,7 @@ class ListSecurityGroupRule(common.NetworkAndComputeLister):
             'id',
             'protocol',
             'remote_ip_prefix',
-            'port_range_min',
+            'port_range',
         )
         if parsed_args.long:
             columns = columns + ('direction', 'ethertype',)
@@ -533,16 +545,13 @@ class ListSecurityGroupRule(common.NetworkAndComputeLister):
         if parsed_args.protocol is not None:
             query['protocol'] = parsed_args.protocol
 
-        rules = list(client.security_group_rules(**query))
-
-        # Reformat the rules to display a port range instead
-        # of just the port range minimum. This maintains
-        # output compatibility with compute.
-        for rule in rules:
-            rule.port_range_min = _format_network_port_range(rule)
+        rules = [
+            self._format_network_security_group_rule(r)
+            for r in client.security_group_rules(**query)
+        ]
 
         return (column_headers,
-                (utils.get_item_properties(
+                (utils.get_dict_properties(
                     s, columns,
                 ) for s in rules))
 
@@ -597,9 +606,9 @@ class ShowSecurityGroupRule(common.NetworkAndComputeShowOne):
     def take_action_network(self, client, parsed_args):
         obj = client.find_security_group_rule(parsed_args.rule,
                                               ignore_missing=False)
-        columns = _get_columns(obj)
+        display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns)
-        return (columns, data)
+        return (display_columns, data)
 
     def take_action_compute(self, client, parsed_args):
         # NOTE(rtheis): Unfortunately, compute does not have an API
