@@ -16,16 +16,36 @@
 """Volume v1 Volume action implementations"""
 
 import argparse
+import logging
+
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 import six
 
-from openstackclient.common import command
-from openstackclient.common import parseractions
-from openstackclient.common import utils
 from openstackclient.i18n import _
 
 
+LOG = logging.getLogger(__name__)
+
+
+def _check_size_arg(args):
+    """Check whether --size option is required or not.
+
+    Require size parameter only in case when snapshot or source
+    volume is not specified.
+    """
+
+    if ((args.snapshot or args.source)
+            is None and args.size is None):
+        msg = _("--size is a required option if snapshot "
+                "or source volume is not specified.")
+        raise exceptions.CommandError(msg)
+
+
 class CreateVolume(command.ShowOne):
-    """Create new volume"""
+    _description = _("Create new volume")
 
     def get_parser(self, prog_name):
         parser = super(CreateVolume, self).get_parser(prog_name)
@@ -37,32 +57,32 @@ class CreateVolume(command.ShowOne):
         parser.add_argument(
             '--size',
             metavar='<size>',
-            required=True,
             type=int,
-            help=_('Volume size in GB'),
+            help=_("Volume size in GB (Required unless --snapshot or "
+                   "--source is specified)"),
         )
         parser.add_argument(
             '--type',
             metavar='<volume-type>',
             help=_("Set the type of volume"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             '--image',
             metavar='<image>',
             help=_('Use <image> as source of volume (name or ID)'),
         )
-        snapshot_group = parser.add_mutually_exclusive_group()
-        snapshot_group.add_argument(
+        source_group.add_argument(
             '--snapshot',
             metavar='<snapshot>',
             help=_('Use <snapshot> as source of volume (name or ID)'),
         )
-        snapshot_group.add_argument(
+        source_group.add_argument(
             '--snapshot-id',
             metavar='<snapshot-id>',
             help=argparse.SUPPRESS,
         )
-        parser.add_argument(
+        source_group.add_argument(
             '--source',
             metavar='<volume>',
             help=_('Volume to clone (name or ID)'),
@@ -94,11 +114,33 @@ class CreateVolume(command.ShowOne):
             help=_('Set a property on this volume '
                    '(repeat option to set multiple properties)'),
         )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable (default)")
+        )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode (default)")
+        )
 
         return parser
 
     def take_action(self, parsed_args):
-
+        _check_size_arg(parsed_args)
         identity_client = self.app.client_manager.identity
         image_client = self.app.client_manager.image
         volume_client = self.app.client_manager.volume
@@ -146,6 +188,22 @@ class CreateVolume(command.ShowOne):
             parsed_args.property,
             image,
         )
+
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+
         # Map 'metadata' column to 'properties'
         volume._info.update(
             {
@@ -158,7 +216,7 @@ class CreateVolume(command.ShowOne):
 
 
 class DeleteVolume(command.Command):
-    """Delete volume(s)"""
+    _description = _("Delete volume(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteVolume, self).get_parser(prog_name)
@@ -170,7 +228,6 @@ class DeleteVolume(command.Command):
         )
         parser.add_argument(
             '--force',
-            dest='force',
             action='store_true',
             default=False,
             help=_('Attempt forced removal of volume(s), regardless of state '
@@ -180,17 +237,31 @@ class DeleteVolume(command.Command):
 
     def take_action(self, parsed_args):
         volume_client = self.app.client_manager.volume
-        for volume in parsed_args.volumes:
-            volume_obj = utils.find_resource(
-                volume_client.volumes, volume)
-            if parsed_args.force:
-                volume_client.volumes.force_delete(volume_obj.id)
-            else:
-                volume_client.volumes.delete(volume_obj.id)
+        result = 0
+
+        for i in parsed_args.volumes:
+            try:
+                volume_obj = utils.find_resource(
+                    volume_client.volumes, i)
+                if parsed_args.force:
+                    volume_client.volumes.force_delete(volume_obj.id)
+                else:
+                    volume_client.volumes.delete(volume_obj.id)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete volume with "
+                            "name or ID '%(volume)s': %(e)s"),
+                          {'volume': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.volumes)
+            msg = (_("%(result)s of %(total)s volumes failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListVolume(command.Lister):
-    """List volumes"""
+    _description = _("List volumes")
 
     def get_parser(self, prog_name):
         parser = super(ListVolume, self).get_parser(prog_name)
@@ -216,6 +287,13 @@ class ListVolume(command.Lister):
             default=False,
             help=_('List additional fields in output'),
         )
+        parser.add_argument(
+            '--limit',
+            type=int,
+            action=parseractions.NonNegativeAction,
+            metavar='<limit>',
+            help=_('Maximum number of volumes to display'),
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -291,7 +369,10 @@ class ListVolume(command.Lister):
             'status': parsed_args.status,
         }
 
-        data = volume_client.volumes.list(search_opts=search_opts)
+        data = volume_client.volumes.list(
+            search_opts=search_opts,
+            limit=parsed_args.limit,
+        )
 
         return (column_headers,
                 (utils.get_item_properties(
@@ -301,8 +382,39 @@ class ListVolume(command.Lister):
                 ) for s in data))
 
 
+class MigrateVolume(command.Command):
+    _description = _("Migrate volume to a new host")
+
+    def get_parser(self, prog_name):
+        parser = super(MigrateVolume, self).get_parser(prog_name)
+        parser.add_argument(
+            'volume',
+            metavar="<volume>",
+            help=_("Volume to migrate (name or ID)")
+        )
+        parser.add_argument(
+            '--host',
+            metavar="<host>",
+            required=True,
+            help=_("Destination host (takes the form: host@backend-name#pool)")
+        )
+        parser.add_argument(
+            '--force-host-copy',
+            action="store_true",
+            help=_("Enable generic host-based force-migration, "
+                   "which bypasses driver optimizations")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        volume_client = self.app.client_manager.volume
+        volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
+        volume_client.volumes.migrate_volume(volume.id, parsed_args.host,
+                                             parsed_args.force_host_copy,)
+
+
 class SetVolume(command.Command):
-    """Set volume properties"""
+    _description = _("Set volume properties")
 
     def get_parser(self, prog_name):
         parser = super(SetVolume, self).get_parser(prog_name)
@@ -334,38 +446,93 @@ class SetVolume(command.Command):
             help=_('Set a property on this volume '
                    '(repeat option to set multiple properties)'),
         )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable")
+        )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode")
+        )
         return parser
 
     def take_action(self, parsed_args):
         volume_client = self.app.client_manager.volume
         volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
 
+        result = 0
         if parsed_args.size:
-            if volume.status != 'available':
-                self.app.log.error(_("Volume is in %s state, it must be "
-                                   "available before size can be extended") %
-                                   volume.status)
-                return
-            if parsed_args.size <= volume.size:
-                self.app.log.error(_("New size must be greater than %s GB") %
-                                   volume.size)
-                return
-            volume_client.volumes.extend(volume.id, parsed_args.size)
-
+            try:
+                if volume.status != 'available':
+                    msg = (_("Volume is in %s state, it must be available "
+                           "before size can be extended") % volume.status)
+                    raise exceptions.CommandError(msg)
+                if parsed_args.size <= volume.size:
+                    msg = (_("New size must be greater than %s GB")
+                           % volume.size)
+                    raise exceptions.CommandError(msg)
+                volume_client.volumes.extend(volume.id, parsed_args.size)
+            except Exception as e:
+                LOG.error(_("Failed to set volume size: %s"), e)
+                result += 1
         if parsed_args.property:
-            volume_client.volumes.set_metadata(volume.id, parsed_args.property)
-
+            try:
+                volume_client.volumes.set_metadata(
+                    volume.id,
+                    parsed_args.property)
+            except Exception as e:
+                LOG.error(_("Failed to set volume property: %s"), e)
+                result += 1
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+                result += 1
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+                result += 1
         kwargs = {}
         if parsed_args.name:
             kwargs['display_name'] = parsed_args.name
         if parsed_args.description:
             kwargs['display_description'] = parsed_args.description
         if kwargs:
-            volume_client.volumes.update(volume.id, **kwargs)
+            try:
+                volume_client.volumes.update(volume.id, **kwargs)
+            except Exception as e:
+                LOG.error(_("Failed to update volume display name "
+                          "or display description: %s"), e)
+                result += 1
+
+        if result > 0:
+            raise exceptions.CommandError(_("One or more of the "
+                                          "set operations failed"))
 
 
 class ShowVolume(command.ShowOne):
-    """Show volume details"""
+    _description = _("Show volume details")
 
     def get_parser(self, prog_name):
         parser = super(ShowVolume, self).get_parser(prog_name)
@@ -395,7 +562,7 @@ class ShowVolume(command.ShowOne):
 
 
 class UnsetVolume(command.Command):
-    """Unset volume properties"""
+    _description = _("Unset volume properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetVolume, self).get_parser(prog_name)
@@ -408,10 +575,8 @@ class UnsetVolume(command.Command):
             '--property',
             metavar='<key>',
             action='append',
-            default=[],
             help=_('Remove a property from volume '
                    '(repeat option to remove multiple properties)'),
-            required=True,
         )
         return parser
 
